#ifndef INFO_HEADER
#define INFO_HEADER

#include "Absyn.h"
#include <memory>
#include <set>
using namespace std;

#include "InfoList.h"

class VarInfo {
public:
	VarInfo(PIdent* ident, TypePtr type) : ident(ident), name(ident->string_), type(type), lineLocation(ident->lineno) {}
	VarInfo(PIdent* ident, Type* type = nullptr) : ident(ident), name(ident->string_), type(type), lineLocation(ident->lineno) {}
	VarInfo(Ar* arg) : VarInfo(arg->pident_, arg->type_) {}
	virtual ~VarInfo() {}

	PIdent *ident;
	string name;
	TypePtr type;
	int lineLocation;

	bool operator<(const VarInfo &other) const {
		return name != other.name ? name < other.name : type < other.type;
	}
};


class FunctionInfo : public VarInfo {
public:
	Block * block;
	vector<VarInfoPtr> arguments;
	weak_ptr<ClassInfo> klass;
	weak_ptr<Binding> binding;
	FunctionInfo(FuncDef *expr, ClassInfoPtr klass = nullptr);
	// FunctionInfo(PIdent *ident, ClassInfoPtr klass = nullptr) : VarInfo(ident), block(NULL), klass(klass) {};
};



class Binding {
public:
	InfoList<VarInfo> variables;
	InfoList<ClassInfo> classes;
	InfoList<FunctionInfo> functions;

	weak_ptr<Binding> parent;
	Binding(BindingPtr parent = nullptr);

	BindingPtr getParent() const { return parent.lock(); };
};

class ClassInfo : public VarInfo, public Binding
{
public:

	ClassInfo(PIdent *ident, BindingPtr parent = nullptr);

	// ClassInfoPtr getParent() const { return dynamic_pointer_cast<ClassInfo>(parent); };
};


class Scope : public Binding {
public:
	BindingPtr currentBinding;
	ClassInfoPtr currentClass;
	FunctionInfoPtr currentFunction;
	Scope();
};

#endif
