#include "ParseError.h"
#include "Info.h"
#include "Printer.h"
#include <sstream>
using namespace std;

ParseError::ParseError(string reason, int line) : runtime_error("ParseError"), line(line)
{
	stringstream ss;
	ss << "ParseError";
	if (line != -1) {
		ss << " at line " << line;
	}
	ss << ": " << reason;
	msg = ss.str();
}

ParseError::ParseError(string reason, Visitable *expr) : runtime_error("ParseError")
{
	stringstream ss;
	ss << "ParseError";
	if (expr) {
		line = expr->lineno;
		ss << " at line " << line;
	}
	ss << ": " << reason;
	if (expr) {
		ss << " Expression: " << PrintAbsyn().print(expr);
	}
	msg = ss.str();
}

RedefinedError::RedefinedError(PIdent *ident, VarInfoPtr orig) {
	stringstream ss;
	ss << "Variable \"" << ident->string_ << "\" at line " << ident->lineno << " redeclared! First declaration at line " << orig->lineLocation;
	msg = ss.str();
	line = ident->lineno;
}

UndefinedError::UndefinedError(PIdent *ident) {
	stringstream ss;
	ss << "Undefined reference to \"" << ident->string_ << "\" at line " << ident->lineno;
	msg = ss.str();
	line = ident->lineno;
}

InvalidTypeError::InvalidTypeError(Type &expected, vector<shared_ptr<Type>> received, Visitable *expr) {
	stringstream ss;
	ss << "Invalid expression type at line " << expr->lineno << ". Expected \"" << expected.printName() << "\", instead received ";
	bool fst = true;
	for (auto i : received) {
		if (fst) fst = false;
		else ss << ", ";
		ss << i->printName();
	}
	ss << ".";
	if (expr) {
		ss << " In: ";
		PrintAbsyn p;
		ss << p.print(expr);
	}
	msg = ss.str();
	line = expr->lineno;
}

InvalidTypeError::InvalidTypeError(Type &expected, vector<Type*> received, Visitable *expr) {
	stringstream ss;
	ss << "Invalid expression type at line " << expr->lineno << ". Expected \"" << expected.printName() << "\", instead received ";
	bool fst = true;
	for (auto i : received) {
		if (fst) fst = false;
		else ss << ", ";
		ss << i->printName();
	}
	ss << ".";
	if (expr) {
		ss << " In: ";
		PrintAbsyn p;
		ss << p.print(expr);
	}
	msg = ss.str();
	line = expr->lineno;
}
